# Copyright: See the LICENSE file.


import os.path

TESTDATA_ROOT = os.path.abspath(os.path.dirname(__file__))
TESTFILE_PATH = os.path.join(TESTDATA_ROOT, 'example.data')
TESTIMAGE_PATH = os.path.join(TESTDATA_ROOT, 'example.jpeg')
